########################################################################
# OSM::Tree::XML::Default

# Shold be the DefaultParser...
# but now there are Problems with Arrays

package OSM::Tree::XML::Default;
use strict;
use warnings;

my $ok;
sub BEGIN
{
  $ok=eval{ use XML::Simple; return 1};
  $ok=0 if($@);
}

sub parse
{
  my $class=shift;
  return undef unless $ok;
  my $data=shift || '';
  my $type=shift || 'data';

  #my $tree=XMLin($data, ForceArray => 1, KeyAttr => [ 'k','id','ref' ], ValueAttr => [ 'v' ], NoSort => 1) if($data ne '');
  my $tree=XMLin($data, ForceArray => 1, KeyAttr => [ 'k','id','ref' ], ValueAttr => [ 'v' ]) if($data ne '');

  return undef unless($tree);

  # ways
  if(exists($tree->{way}) && ref($tree->{way}) eq 'HASH')
  {
    # normalize Tree if only one Way exists
    if(exists($tree->{way}->{id}))
    {
      my $way=$tree->{way};
      $tree->{way}={};
      $tree->{way}->{$way->{id}}=$way;
    }

    __cleanup_tags($tree->{way});
    for my $id (keys(%{$tree->{way}}))
    {
      my $way=$tree->{way}->{$id};
      $way->{nd}=[keys(%{$way->{nd}})];
    }
  }
  else
  { $tree->{way}={};  }

  # nodes
  if(exists($tree->{node}) && ref($tree->{node}) eq 'HASH')
  {
    # normalize Tree if only one Node exists
    if(exists($tree->{node}->{id}))
    {
      my $node=$tree->{node};
      $tree->{node}={};
      $tree->{node}->{$node->{id}}=$node;
    }

    __cleanup_tags($tree->{node});
  }
  else
  { $tree->{node}={}; }

  # relations
  if(exists($tree->{relation}) && ref($tree->{relation}) eq 'HASH')
  {
    __cleanup_tags($tree->{relation});

    # normalize Tree if only one relation exists
    if(exists($tree->{relation}->{id}))
    {
      my $rel=$tree->{relation};
      $tree->{relation}={};
      $tree->{relation}->{$rel->{id}}=$rel;
    }
  }
  else
  { $tree->{relation}={}; }


  # members
  if(exists($tree->{member}) && ref($tree->{member}) eq 'HASH')
  {
    # normalize Tree if only one relation exists
    if(exists($tree->{member}->{id}))
    {
      my $mem=$tree->{member};
      $tree->{member}={};
      $tree->{member}->{$mem->{id}}=$mem;
    }
  }
  else
  { $tree->{member}={}; }

  #print Dumper($tree); exit;

  return $tree;
}

sub __cleanup_tags
{
  my $ref=shift;
  for my $id ( keys(%{$ref}) )
  {
    my $elm=$ref->{$id};
    if( ref($elm->{tag}) eq 'HASH')
    {
      for my $tag (keys(%{$elm->{tag}}))
      { $elm->{tag}->{$tag}=$elm->{tag}->{$tag}->{v}; }
    }
    else
    { $ref->{$id}->{tag}={}; }
  }
}

1;
